@@ -33,7 +33,6 @@ import re
 from itertools import chain
 from typing import (cast, Dict, FrozenSet, Iterable, List, Mapping,
                     ClassVar, Optional, Sequence, Set, Tuple, Union)
-
 from google.api import annotations_pb2      # type: ignore
 from google.api import client_pb2
 from google.api import field_behavior_pb2
@@ -56,12 +55,15 @@ class Field:
     )
     oneof: Optional[str] = None
 
-    # Arbitrary cap set via heuristic rule of thumb.
-    MAX_MOCK_DEPTH: int = 20
-
     def __getattr__(self, name):
         return getattr(self.field_pb, name)
 
+    def __hash__(self):
+        # The only sense in which it is meaningful to say a field is equal to
+        # another field is if they are the same, i.e. they live in the same
+        # message type under the same moniker, i.e. they have the same id.
+        return id(self)
+
     @property
     def name(self) -> str:
         """Used to prevent collisions with python keywords"""
@@ -88,17 +90,16 @@ class Field:
 
     @utils.cached_property
     def mock_value(self) -> str:
-        depth = 0
+        visited_fields: Set["Field"] = set()
         stack = [self]
         answer = "{}"
         while stack:
             expr = stack.pop()
-            answer = answer.format(expr.inner_mock(stack, depth))
-            depth += 1
+            answer = answer.format(expr.inner_mock(stack, visited_fields))
 
         return answer
 
-    def inner_mock(self, stack, depth):
+    def inner_mock(self, stack, visited_fields):
         """Return a repr of a valid, usually truthy mock value."""
         # For primitives, send a truthy value computed from the
         # field name.
@@ -132,10 +133,11 @@ class Field:
                 and isinstance(self.type, MessageType)
                 and len(self.type.fields)
                 # Nested message types need to terminate eventually
-                and depth < self.MAX_MOCK_DEPTH
+                and self not in visited_fields
         ):
             sub = next(iter(self.type.fields.values()))
             stack.append(sub)
+            visited_fields.add(self)
             # Don't do the recursive rendering here, just set up
             # where the nested value should go with the double {}.
             answer = f'{self.type.ident}({sub.name}={{}})'
@@ -217,7 +219,12 @@ class Field:
         raise TypeError(f'Unrecognized protobuf type: {self.field_pb.type}. '
                         'This code should not be reachable; please file a bug.')
 
-    def with_context(self, *, collisions: FrozenSet[str]) -> 'Field':
+    def with_context(
+            self,
+            *,
+            collisions: FrozenSet[str],
+            visited_messages: FrozenSet["MessageType"],
+    ) -> 'Field':
         """Return a derivative of this field with the provided context.
 
         This method is used to address naming collisions. The returned
@@ -228,7 +235,8 @@ class Field:
             self,
             message=self.message.with_context(
                 collisions=collisions,
-                skip_fields=True,
+                skip_fields=self.message in visited_messages,
+                visited_messages=visited_messages,
             ) if self.message else None,
             enum=self.enum.with_context(collisions=collisions)
             if self.enum else None,
@@ -305,6 +313,15 @@ class MessageType:
 
         return tuple(types)
 
+    @utils.cached_property
+    def recursive_fields(self) -> FrozenSet[Field]:
+        return frozenset(chain(
+            self.fields.values(),
+            (field
+             for t in self.recursive_field_types if isinstance(t, MessageType)
+             for field in t.fields.values()),
+        ))
+
     @property
     def map(self) -> bool:
         """Return True if the given message is a map, False otherwise."""
@@ -392,7 +409,10 @@ class MessageType:
 
         # Base case: If this is the last field in the path, return it outright.
         if len(field_path) == 1:
-            return cursor.with_context(collisions=collisions)
+            return cursor.with_context(
+                collisions=collisions,
+                visited_messages=frozenset({self}),
+            )
 
         # Sanity check: If cursor is a repeated field, then raise an exception.
         # Repeated fields are only permitted in the terminal position.
@@ -419,6 +439,7 @@ class MessageType:
     def with_context(self, *,
                      collisions: FrozenSet[str],
                      skip_fields: bool = False,
+                     visited_messages: FrozenSet["MessageType"] = frozenset(),
                      ) -> 'MessageType':
         """Return a derivative of this message with the provided context.
 
@@ -430,10 +451,14 @@ class MessageType:
         underlying fields. This provides for an "exit" in the case of circular
         references.
         """
+        visited_messages = visited_messages | {self}
         return dataclasses.replace(
             self,
             fields=collections.OrderedDict(
-                (k, v.with_context(collisions=collisions))
+                (k, v.with_context(
+                    collisions=collisions,
+                    visited_messages=visited_messages
+                ))
                 for k, v in self.fields.items()
             ) if not skip_fields else self.fields,
             nested_enums=collections.OrderedDict(
@@ -443,7 +468,9 @@ class MessageType:
             nested_messages=collections.OrderedDict(
                 (k, v.with_context(
                     collisions=collisions,
-                    skip_fields=skip_fields,))
+                    skip_fields=skip_fields,
+                    visited_messages=visited_messages,
+                ))
                 for k, v in self.nested_messages.items()),
             meta=self.meta.with_context(collisions=collisions),
         )
@@ -860,6 +887,13 @@ class CommonResource:
     type_name: str
     pattern: str
 
+    @classmethod
+    def build(cls, resource: resource_pb2.ResourceDescriptor):
+        return cls(
+            type_name=resource.type,
+            pattern=next(iter(resource.pattern))
+        )
+
     @utils.cached_property
     def message_type(self):
         message_pb = descriptor_pb2.DescriptorProto()
@@ -880,33 +914,37 @@ class Service:
     """Description of a service (defined with the ``service`` keyword)."""
     service_pb: descriptor_pb2.ServiceDescriptorProto
     methods: Mapping[str, Method]
+    # N.B.: visible_resources is intended to be a read-only view
+    # whose backing store is owned by the API.
+    # This is represented by a types.MappingProxyType instance.
+    visible_resources: Mapping[str, MessageType]
     meta: metadata.Metadata = dataclasses.field(
         default_factory=metadata.Metadata,
     )
 
-    common_resources: ClassVar[Sequence[CommonResource]] = dataclasses.field(
-        default=(
-            CommonResource(
+    common_resources: ClassVar[Mapping[str, CommonResource]] = dataclasses.field(
+        default={
+            "cloudresourcemanager.googleapis.com/Project": CommonResource(
                 "cloudresourcemanager.googleapis.com/Project",
                 "projects/{project}",
             ),
-            CommonResource(
+            "cloudresourcemanager.googleapis.com/Organization": CommonResource(
                 "cloudresourcemanager.googleapis.com/Organization",
                 "organizations/{organization}",
             ),
-            CommonResource(
+            "cloudresourcemanager.googleapis.com/Folder": CommonResource(
                 "cloudresourcemanager.googleapis.com/Folder",
                 "folders/{folder}",
             ),
-            CommonResource(
+            "cloudbilling.googleapis.com/BillingAccount": CommonResource(
                 "cloudbilling.googleapis.com/BillingAccount",
                 "billingAccounts/{billing_account}",
             ),
-            CommonResource(
+            "locations.googleapis.com/Location": CommonResource(
                 "locations.googleapis.com/Location",
                 "projects/{project}/locations/{location}",
             ),
-        ),
+        },
         init=False,
         compare=False,
     )
@@ -1021,6 +1059,17 @@ class Service:
                 if type_.resource_path:
                     yield type_
 
+        def gen_indirect_resources_used(message):
+            for field in message.recursive_fields:
+                resource = field.options.Extensions[
+                    resource_pb2.resource_reference]
+                resource_type = resource.type or resource.child_type
+                # The common resources are defined (and rendered) explicitly
+                # by separate logic, and the resource definitions are never
+                # visible in any of the APIs file descriptor protos.
+                if resource_type and resource_type not in self.common_resources:
+                    yield self.visible_resources[resource_type]
+
         return frozenset(
             msg
             for method in self.methods.values()
@@ -1029,6 +1078,10 @@ class Service:
                 gen_resources(
                     method.lro.response_type if method.lro else method.output
                 ),
+                gen_indirect_resources_used(method.input),
+                gen_indirect_resources_used(
+                    method.lro.response_type if method.lro else method.output
+                ),
             )
         )
 
