@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import distutils
+import os
 import pytest
 from google import showcase
 
@@ -47,39 +49,39 @@ def test_pagination_pages(echo):
                        for i in text.split(' ')]
 
 
-@pytest.mark.asyncio
-async def test_pagination_async(async_echo):
-    text = 'The hail in Wales falls mainly on the snails.'
-    results = []
-    async for i in await async_echo.paged_expand({
-        'content': text,
-        'page_size': 3,
-    }):
-        results.append(i)
+if distutils.util.strtobool(os.environ.get("GAPIC_PYTHON_ASYNC", "true")):
+    @pytest.mark.asyncio
+    async def test_pagination_async(async_echo):
+        text = 'The hail in Wales falls mainly on the snails.'
+        results = []
+        async for i in await async_echo.paged_expand({
+            'content': text,
+            'page_size': 3,
+        }):
+            results.append(i)
 
-    assert len(results) == 9
-    assert results == [showcase.EchoResponse(content=i)
-                       for i in text.split(' ')]
+        assert len(results) == 9
+        assert results == [showcase.EchoResponse(content=i)
+                           for i in text.split(' ')]
 
+    @pytest.mark.asyncio
+    async def test_pagination_pages_async(async_echo):
+        text = "The hail in Wales falls mainly on the snails."
+        page_results = []
+        async for page in (await async_echo.paged_expand({
+            'content': text,
+            'page_size': 3,
+        })).pages:
+            page_results.append(page)
 
-@pytest.mark.asyncio
-async def test_pagination_pages_async(async_echo):
-    text = "The hail in Wales falls mainly on the snails."
-    page_results = []
-    async for page in (await async_echo.paged_expand({
-        'content': text,
-        'page_size': 3,
-    })).pages:
-        page_results.append(page)
+        assert len(page_results) == 3
+        assert not page_results[-1].next_page_token
 
-    assert len(page_results) == 3
-    assert not page_results[-1].next_page_token
+        # The monolithic surface uses a wrapper type that needs an explicit property
+        # for a 'raw_page': we need to duplicate that interface, even though the
+        # architecture is different.
+        assert page_results[0].raw_page is page_results[0]
 
-    # The monolithic surface uses a wrapper type that needs an explicit property
-    # for a 'raw_page': we need to duplicate that interface, even though the
-    # architecture is different.
-    assert page_results[0].raw_page is page_results[0]
-
-    results = [r for p in page_results for r in p.responses]
-    assert results == [showcase.EchoResponse(content=i)
-                       for i in text.split(' ')]
+        results = [r for p in page_results for r in p.responses]
+        assert results == [showcase.EchoResponse(content=i)
+                           for i in text.split(' ')]
