@@ -18,12 +18,12 @@ from gapic.utils import filename
 def test_to_valid_filename():
     assert filename.to_valid_filename('foo bar.py') == 'foo-bar.py'
     assert filename.to_valid_filename('FOO') == 'foo'
-    assert filename.to_valid_filename('nom$&nom@nom.py') == 'nom-nom-nom.py'
+    assert filename.to_valid_filename('nom%&nom@nom.py') == 'nom-nom-nom.py'
     assert filename.to_valid_filename('num_bear.py') == 'num_bear.py'
 
 
 def test_to_valid_module_name():
     assert filename.to_valid_module_name('foo bar.py') == 'foo_bar.py'
     assert filename.to_valid_module_name('FOO') == 'foo'
-    assert filename.to_valid_module_name('nom$&nom.py') == 'nom_nom.py'
+    assert filename.to_valid_module_name('nom%&nom.py') == 'nom_nom.py'
     assert filename.to_valid_module_name('num_bear.py') == 'num_bear.py'
